package de.opalpiraten.scenepainter.gui;
import java.awt.*;
import java.io.*;    
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;  
import javax.swing.event.*;
import java.beans.*;
import java.net.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;
import com.threerings.cast.*;
import com.threerings.media.image.*;
import com.threerings.util.*;

/**
 * This Editor is the GUI frontend for the ComponentSceneElement class
 */
public class ComponentSceneElementEditor extends JPanel implements 
ActionListener, ChangeListener, TreeSelectionListener, SceneListener {

  /**
   * The rootnode name in the rsrctree
   */
  private static final String RSRCROOT="Components";
  
  /**
   * The rootnodename of the colortree
   */
  private static final String COLORROOT="Colors";
  
  /**
   * Holds the resourcenames
   */
  private JTree rsrctree;
  
  /**
   * Holds the colornames and classes
   */
  private JTree coltree;
  
  /**
   * For Setting the orientation of the Composite
   */
  //private JSlider dir = new JSlider(0,DirectionCodes.DIRECTION_COUNT-1);
  private JSpinner dir = new JSpinner(new SpinnerListModel(DirectionUtil.getDirectionNames()));
  
  /**
   * For setting the Frame index (this is a made up number. There needs
   * to be a way to recover the real number, but it is not a pressing matter).
   */
  //private JSlider index = new JSlider(0,12);
  private JSpinner index = new JSpinner(new SpinnerNumberModel());
  
  /**
   * For Setting the action
   */
  private JComboBox action = new JComboBox();
  
  /**
   * For selecting which component to recolor
   */
  private JComboBox comsel = new JComboBox();
  
  /**
   * Flag signalling, that Listeners should bail out. This is required, so
   * setScene does not become reentrant.
   */
  private boolean noupdate=false;
  
  /**
   * Reference to the scene showing
   */
  private Scene scene;
  
  /**
   * Reference to the project
   */
  private Project project;
  
  /**
   * Reference to the container this editor resides in.
   */
  private SceneElementEditor parent;

  public ComponentSceneElementEditor(SceneElementEditor see) {
    parent = see;
    DefaultMutableTreeNode top_rsrc = new DefaultMutableTreeNode(RSRCROOT);
    DefaultMutableTreeNode top_col = new DefaultMutableTreeNode(COLORROOT);
    rsrctree = new JTree(top_rsrc);
    coltree = new JTree(new DefaultMutableTreeNode("Colors"));
    coltree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
    rsrctree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

    String[] names = Core.nenyabrowser.getComponentNames();
    for (int i=0;i<names.length;i++) Util.sortIntoTree(top_rsrc,names[i].split("/"));
    rsrctree.expandPath(new TreePath(top_rsrc));
    
    JScrollPane jsp0 = new JScrollPane(rsrctree);
    JScrollPane jsp1 = new JScrollPane(coltree);
    JPanel colpane = new JPanel();
    colpane.setLayout(new BoxLayout(colpane,BoxLayout.Y_AXIS));
    
    comsel.addItem(" ");
    colpane.setPreferredSize(colpane.getPreferredSize());
    colpane.add(jsp1);
    colpane.add(comsel);
    JTabbedPane split = new JTabbedPane(JTabbedPane.BOTTOM);
    split.add(Dict.get("gui.componentsceneelementeditor.split.components"),jsp0);
    split.add(Dict.get("gui.componentsceneelementeditor.split.colors"),colpane);
    split.setPreferredSize(split.getPreferredSize());
    
    Iterator it = Core.nenyabrowser.getComponentRepository().enumerateActionSequences();
    Vector v = new Vector();
    while (it.hasNext()) {
      v.add( ((ActionSequence) (it.next())).name );
    }
    String[] actnames= new String[v.size()];
    v.copyInto(actnames);
    Arrays.sort(actnames);
    action = new JComboBox();
    action.setEditable(true);
    for (int i=actnames.length-1;i>=0;i--) {
      action.addItem(actnames[i]);
    }
    action.setPreferredSize(action.getPreferredSize());

    JPanel adjust = new JPanel();
    adjust.setLayout(new GridLayout(3,2));
    adjust.add(new JLabel(Dict.get("gui.componentsceneelementeditor.adjust.action")));
    adjust.add(action);
    adjust.add(new JLabel(Dict.get("gui.componentsceneelementeditor.adjust.dir")));
    adjust.add(dir);
    adjust.add(new JLabel(Dict.get("gui.componentsceneelementeditor.adjust.index")));
    adjust.add(index);
    adjust.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.componentsceneelementeditor.adjust")));
    

    GridBagLayout gbl = new GridBagLayout();
    setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();

    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=2;
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(split,gbc);
    add(split);
    
    gbc.anchor=GridBagConstraints.CENTER;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=1;
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(adjust,gbc);
    add(adjust);
    
    SpinnerNumberModel snm = (SpinnerNumberModel)index.getModel();
    snm.setMinimum(new Integer(0));
    
    action.addActionListener(this);
    dir.addChangeListener(this);
    index.addChangeListener(this);
    rsrctree.addTreeSelectionListener(this);
    coltree.addTreeSelectionListener(this);
    
    
  }
  
  public void actionPerformed(ActionEvent e) {
    if (noupdate) return;
    Object src = e.getSource();
    
    if (src==action) {
      try {
        ComponentSceneElement cse = (ComponentSceneElement)scene.getSelected();
        cse.setAction(action.getSelectedItem().toString());
        scene.replace(scene.getSelectedIndex(),cse);
      }
      catch (Exception exp) {
        // Lazy sanity checking
        //exp.printStackTrace();
      }
    }
  }
  
  public void replaceScene(Scene s) {
    if (scene!=null) scene.removeSceneListener(this);
    scene=s;
    scene.addSceneListener(this);
    sceneChanged(s);
  }
  
  public void sceneChanged (Scene s) {
    noupdate=true;  // Make sure nothing fires, while their datamodel is set.

    try {
      
      ComponentSceneElement cse = (ComponentSceneElement)s.getSelected();
      index.setValue(new Integer(cse.getIndex()));
      dir.setValue(DirectionUtil.getDirectionNames()[cse.getOrientation()]);
      
      action.setSelectedItem(cse.getAction());

      
      Hashtable hash = cse.getResources();
      Enumeration e = hash.keys();
      Object tmp = comsel.getSelectedItem();
      comsel.removeAllItems();
      TreePath[] old =rsrctree.getSelectionPaths();
      if (old!=null) rsrctree.removeSelectionPaths(old);
      
      // Travel through the resourcetree and select the nodes
      while (e.hasMoreElements()) {
        String rsrc = (String)e.nextElement();
        comsel.addItem(rsrc);
        String[] elem = rsrc.toString().split("/");
    
        DefaultTreeModel dtm = (DefaultTreeModel)rsrctree.getModel();
        TreeNode trav = (TreeNode) dtm.getRoot();
        for (int x=0;x<elem.length;x++) {
          int z = dtm.getChildCount(trav);
          for (int y=0;y<z;y++) {
            TreeNode tmpnode = (TreeNode) dtm.getChild(trav,y);
            if (elem[x].equals(tmpnode.toString())) {
              trav=tmpnode;
              break;
            }
          }
        }
        //System.err.println(trav);
        rsrctree.addSelectionPath(new TreePath(dtm.getPathToRoot(trav)));
      }
      
      comsel.setSelectedItem(tmp);
    }
    catch (Exception e) {
      // Lazy about sanity checks
    }
    
    noupdate=false;
  }
  
  
  
  public void replaceProject(Project p) {
    // CharacterClasses do not always seem to know, which colorizations
    // they support, so simply show them all.
    ColorPository cpos = p.getColorPository();
    DefaultMutableTreeNode top = new DefaultMutableTreeNode(COLORROOT);
    Iterator clazzes = cpos.enumerateClasses();
    Vector v = new Vector();
    while (clazzes.hasNext()) {
      ColorPository.ClassRecord clazz = (ColorPository.ClassRecord)clazzes.next();
      ColorPository.ColorRecord[] crec= cpos.enumerateColors(clazz.name);
      for (int i=0;i<crec.length;i++) {
        v.add(clazz.name+"/"+crec[i].name);
      }
    }
    String tmp[] = new String[v.size()];
    v.copyInto(tmp);
    Arrays.sort(tmp);
    for (int i=0;i<tmp.length;i++)  {
      Util.sortIntoTree(top,tmp[i].split("/"));
    }
    coltree.setModel(new DefaultTreeModel(top));
    project=p;
  }
  
  public void stateChanged(ChangeEvent e) {
    if (noupdate) return;
    Object src = e.getSource();
    if (src==dir) {
      try {
        ComponentSceneElement cse = (ComponentSceneElement)scene.getSelected();
        //cse.setOrientation(((Integer)dir.getValue()).intValue());
        cse.setOrientation(DirectionUtil.fromString(dir.getValue().toString()));
        scene.replace(scene.getSelectedIndex(),cse);
      }
      catch (Exception exp) {
        // Lazy checking
      }
    }
    
    if (src==index) {
      try {
        ComponentSceneElement cse = (ComponentSceneElement)scene.getSelected();
        cse.setIndex(((Integer)index.getValue()).intValue());
        scene.replace(scene.getSelectedIndex(),cse);
      }
      catch (Exception exp) {
        // Lazy checking
      }
    }
    
  }
  
  public void valueChanged(TreeSelectionEvent e) {
    if (noupdate) return;
    Object src = e.getSource();

    if (src==coltree) {
      try {
        // Alyways start with the selected scene (if any ... throw up if none)
        ComponentSceneElement cse = (ComponentSceneElement)scene.getSelected();
        Hashtable cse_res = cse.getResources();
        
        // Whats the last path selected?
        String cri = (String)comsel.getSelectedItem();
        if (cri==null) return;
        
        Hashtable colz = new Hashtable();
        TreePath tp[] = coltree.getSelectionPaths();
        for (int i=0;i<tp.length;i++) {
          Object [] tmp = tp[i].getPath();
          if (tmp.length!=3) break;
          colz.put(tmp[1].toString(),tmp[2].toString());
        }
        
        cse_res.put(cri,colz);
        cse.setResources(cse_res);
        // shove back again...
        scene.replace(scene.getSelectedIndex(),cse);
      }
      catch (Exception exp) {
         //exp.printStackTrace();
        // Be lazy about sanity checks
      }
    }

    if (src==rsrctree) {
      try {
        // Put the selection in here
        Hashtable hash = new Hashtable();
        
        // See if we should cary over colors from previous selections
        Hashtable prev=null;
        
        comsel.removeAllItems();
        
        try {
          ComponentSceneElement cse = (ComponentSceneElement)scene.getSelected();
          prev=cse.getResources();
          if (prev==null) prev=new Hashtable();
        }
        catch (Exception exp) {
          // No previous selections to worry about
          prev=new Hashtable();
          
        }
        
        TreePath sel[] = rsrctree.getSelectionPaths();
        for (int x=0;x<sel.length;x++) {
          String resource=getCRI(sel[x]);
          comsel.addItem(resource);
          Object obj = prev.get(resource);
          // Merge with previous selections
          if (obj==null) hash.put(resource,new Hashtable());
          else hash.put(resource,prev.get(resource));
        }
        comsel.setSelectedItem(getCRI(null));
        
        
        ComponentSceneElement cse = new ComponentSceneElement();
        cse.setProject(project);
        cse.setAction(action.getSelectedItem().toString());
        cse.setResources(hash);
        cse.setIndex(((Integer)index.getValue()).intValue());
        //cse.setOrientation(((Integer)dir.getValue()).intValue());
        cse.setOrientation(DirectionUtil.fromString(dir.getValue().toString()));
        
        if (parent.isCreating() || scene.getSelectedIndex()<0) {
          scene.add(cse);
          scene.setSelectedIndex(0);
        }
        else {
          cse.setPosition(scene.getSelected().getPosition());
          cse.setAlpha(scene.getSelected().getAlpha());
          scene.replace(scene.getSelectedIndex(),cse);
        }
      }
      catch (Exception exp) {
        // Be lazy about sanity checks
       // exp.printStackTrace();
       //System.err.println(exp);
      }
    }
    
  }
  
  /**
   * Return the ComponentResourceIdentifier (path) of a selected component
   * in the tree
   * @param tp either a treepath to convert to a CRI or null to get the
   * CRI of the last path selected by the user
   * @return a CRI, that can be used by NenyaBrowser.getComponentId(String cri)
   * or null if the treepath could not be mapped to a CRI.
   */
  private String getCRI(TreePath tpath) {
    TreePath tp;
    if (tpath==null) {
      TreePath sel[] = rsrctree.getSelectionPaths();
      if (sel==null || sel.length==0) return null;
      tp = sel[sel.length-1];
    } 
    else tp =tpath;
    TreeNode tn  =(TreeNode) tp.getLastPathComponent();
    
    if (!tn.isLeaf()) return null;
    StringBuffer sb = new StringBuffer();
    Object path[]=tp.getPath();
    for (int i=1;i<path.length;i++) {
      sb.append(path[i]);
      if (i<path.length-1) sb.append("/");
    }
    return sb.toString();
  }
  
  
}
