package de.opalpiraten.scenepainter.tools;
import java.util.*;
import java.util.jar.*;
import java.net.*;
import java.io.*;
import de.opalpiraten.scenepainter.sys.*;

/**
 * This class is responsible for loading the plugins found in the tools
 * directory
 */
public class ToolLoader extends ClassLoader {

  /**
   * Name of the directory containing the tool jars
   */
  public static final File TOOLDIR = new File("tools");
  
  /**
   * Constructor
   */
  public ToolLoader() {
    if (!TOOLDIR.exists()) TOOLDIR.mkdirs();
  }
  
  
  /**
   * Search the JAR files in the tooldir instantiate and return all classes
   * implementing the Tool interface. For every Jar, there is also an own
   * instance of the Classloader, that can be be used for loading further
   * resources from that jar.
   * @return an instance of all clases found implementing 
   * de.opalpiraten.scenepainter.tools.Tool
   */
  public Tool[] getTools() {
    Tool ret[] = new Tool[0];
    
    Vector tmp = new Vector();
    File[] tools = TOOLDIR.listFiles(new SuffixFilter(SuffixFilter.JAR,false));
    URL[] u = new URL[1];
    String[] names;
    Arrays.sort(tools);
    for (int x=0;x<tools.length;x++) {
      try {
        u[0]=tools[x].toURL();
        URLClassLoader loader = new URLClassLoader(u);
        Dict.addBundle(loader);
        names = getStartable(tools[x]);
        for (int y=0;y<names.length;y++) {
          tmp.add(Class.forName(names[y],true,loader).newInstance());
        }
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    ret = new Tool[tmp.size()];
    tmp.copyInto(ret);
    return ret;
  }
  
  /**
   * Open a jar file and find the names of the classes, that are startable
   * @param f the jarfile to open
   * @return The (fully qualified) names of the classes implementing the
   * de.opalpiraten.scenepainter.tools.Tool interface.
   */
  private String[] getStartable(File f) {
    String[] ret = new String[0];
    
    try {
      JarFile plugin = new JarFile(f);
      Enumeration e = plugin.entries();
      Class startable = Class.forName("de.opalpiraten.scenepainter.tools.Tool");
      Vector tmp = new Vector(); 
      while(e.hasMoreElements()) {                                      
        JarEntry je = (JarEntry)e.nextElement();
        if(je.getName().toUpperCase().endsWith(".CLASS")) {
          byte[] buf = new byte[(int)je.getSize()];
          BufferedInputStream ins;
          ins = new BufferedInputStream(plugin.getInputStream(je));
          ins.read(buf);
          try {
            Class cl = defineClass(null,buf,0,buf.length);
            if (startable.isAssignableFrom(cl)) tmp.add(cl.getName());
          }
          catch (NoClassDefFoundError err) {
            // If not a proper class, it is not startable -> ignore
            //err.printStackTrace();
            //System.err.println("Invalid class in "+f.getName()+": "+ err.getMessage());
          }
        }
      }
      ret = new String[tmp.size()];
      tmp.copyInto(ret);
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
    return ret;
  }
}
